.SH FORMAT

File format is described in the beginning of the script itself.

.SH "SEE ALSO"
xorriso(1), jigdo-lite(1), jigit-mkimage(1)

.SH AUTHOR
Written by George Danchev <danchev@spnet.net> and Thomas Schmitt <scdbackup@gmx.net>
